@@ -19,6 +19,17 @@
 
 
 
+stackedRange <- function(X, INDEX)
+{
+    X[is.na(X)] <- 0 # missing values treated as 0
+    pos <- if (any(ID <- (X > 0)))
+               tapply(X[ID], INDEX[ID, drop = TRUE], sum, na.rm = TRUE)
+           else 0
+    neg <- if (any(ID <- (X < 0)))
+               tapply(X[ID], INDEX[ID, drop = TRUE], sum, na.rm = TRUE)
+           else 0
+    range(pos, neg, finite = TRUE)
+}
 
 
 prepanel.default.bwplot <-
@@ -31,7 +42,7 @@ prepanel.default.bwplot <-
     ## bwplot family, namely bwplot, dotplot, stripplot and
     ## barchart. For all but barchart, this is simply a question of
     ## getting the ranges. For stacked barcharts, things are slightly
-    ## complicated.
+    ## complicated: see stackedRange() above.
 
     if (any(!is.na(x) & !is.na(y)))
     {
@@ -42,21 +53,7 @@ prepanel.default.bwplot <-
                 if (missing(nlevels)) nlevels <- length(unique(y))
                 y <- factor(y, levels = 1:nlevels)
             }
-            list(xlim =
-                 if (stack) {
-                     foo1 <-
-                         if (any(x > 0))
-                             range(tapply(x[x > 0], y[x > 0, drop = TRUE], sum, na.rm = TRUE), finite = TRUE) 
-                         else 0
-                     foo2 <-
-                         if (any(x < 0))
-                             range(tapply(x[x < 0], y[x < 0, drop = TRUE], sum, na.rm = TRUE), finite = TRUE) 
-                         else 0
-                     range(foo1, foo2)
-                 }
-                 ## else if (is.numeric(x)) range(x, origin, finite = TRUE)
-                 ## else levels(x),
-                 else scale.limits(c(x, origin)),
+            list(xlim = if (stack) stackedRange(x, y) else scale_limits(c(x, origin)),
                  ylim = levels(y),
                  yat = sort(unique(as.numeric(y))),
                  dx = 1,
@@ -71,21 +68,7 @@ prepanel.default.bwplot <-
             }
             list(xlim = levels(x),
                  xat = sort(unique(as.numeric(x))),
-                 ylim =
-                 if (stack) {
-                     foo1 <-
-                         if (any(y > 0))
-                             range(tapply(y[y > 0], x[y > 0], sum, na.rm = TRUE), finite = TRUE)
-                         else 0
-                     foo2 <-
-                         if (any(y < 0))
-                             range(tapply(y[y < 0], x[y < 0], sum, na.rm = TRUE), finite = TRUE)
-                         else 0
-                     range(foo1, foo2)
-                 }
-                 ## else if (is.numeric(y)) range(y, origin, finite = TRUE)
-                 ## else levels(y),
-                 else scale.limits(c(y, origin)),
+                 ylim = if (stack) stackedRange(y, x) else scale_limits(c(y, origin)),
                  dx = 1,
                  dy = 1)
         }
@@ -163,8 +146,7 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
 
             panel.rect(x = rep(origin, length(y)),
                        y = y,
@@ -175,10 +157,7 @@ panel.barchart <-
                        just = c("left", "centre"),
                        identifier = identifier)
         }
-
-        ## grouped, with stacked bars
-
-        else if (stack)
+        else if (stack) # grouped, with stacked bars
         {
             if (!is.null(origin) && origin != 0)
                 warning("'origin' forced to 0 for stacked bars")
@@ -193,15 +172,15 @@ panel.barchart <-
             lty <- rep(lty, length.out = nvals)
             lwd <- rep(lwd, length.out = nvals)
 
-            height <- box.width # box.ratio / (1 + box.ratio)
+            height <- # box.ratio / (1 + box.ratio) by default
+                rep(box.width, length.out = length(x))
 
             if (reference)
                 panel.abline(v = origin,
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
 
             for (i in unique(y))
             {
@@ -216,11 +195,10 @@ panel.barchart <-
                                border = border[groups[ok][ord][pos]],
                                lty = lty[groups[ok][ord][pos]],
                                lwd = lwd[groups[ok][ord][pos]],
-                               height = rep(height, nok),
+                               height = height[ok][ord][pos],
                                width = x[ok][ord][pos],
                                just = c("left", "centre"),
-                               identifier = paste(identifier, "pos", i,
-                                 sep = "."))
+                               identifier = paste(identifier, "pos", i, sep = "."))
                 neg <- x[ok][ord] < 0
                 nok <- sum(neg, na.rm = TRUE)
                 if (nok > 0)
@@ -230,17 +208,13 @@ panel.barchart <-
                                border = border[groups[ok][ord][neg]],
                                lty = lty[groups[ok][ord][neg]],
                                lwd = lwd[groups[ok][ord][neg]],
-                               height = rep(height, nok),
+                               height = height[ok][ord][neg],
                                width = x[ok][ord][neg],
                                just = c("left", "centre"),
-                               identifier = paste(identifier, "neg", i,
-                                 sep = "."))
+                               identifier = paste(identifier, "neg", i, sep = "."))
             }
         }
-
-        ## grouped, with side by side bars
-
-        else
+        else # grouped, with side by side bars
         {
             if (is.null(origin))
             {
@@ -263,8 +237,7 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
             for (i in unique(y))
             {
                 ok <- y == i
@@ -275,19 +248,17 @@ panel.barchart <-
                            border = border[groups[ok]],
                            lty = lty[groups[ok]],
                            lwd = lwd[groups[ok]],
-                           height = rep(height, nok),
+                           height = rep(height, nok), # rep(height[i], nok),
                            width = x[ok] - origin,
                            just = c("left", "centre"),
-                           identifier = paste(identifier, "y", i,
-                               sep = "."))
+                           identifier = paste(identifier, "y", i, sep = "."))
             }
         }
     }
-    
-    ## if not horizontal
 
-    else
+    else # if not horizontal
     {
+        ## No grouping
         if (is.null(groups))
         {
             if (is.null(origin))
@@ -295,15 +266,14 @@ panel.barchart <-
                 origin <- current.panel.limits()$ylim[1]
                 reference <- FALSE
             }
-            width <- box.width # box.ratio/(1+box.ratio)
+            width <- box.width # box.ratio / (1 + box.ratio)
 
             if (reference)
                 panel.abline(h = origin,
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
 
             panel.rect(x = x,
                        y = rep(origin, length(x)),
@@ -314,31 +284,24 @@ panel.barchart <-
                        just = c("centre", "bottom"),
                        identifier = identifier)
         }
-        else if (stack)
+        else if (stack) # grouped, with stacked bars
         {
-
             if (!is.null(origin) && origin != 0)
                 warning("'origin' forced to 0 for stacked bars")
-
-##             vals <- seq_len(nlevels(groups))
-##             groups <- as.numeric(groupSub(groups, ...))
-##             ## vals <- sort(unique(groups))
-##             nvals <- length(vals)
-
             col <- rep(col, length.out = nvals)
             border <- rep(border, length.out = nvals)
             lty <- rep(lty, length.out = nvals)
             lwd <- rep(lwd, length.out = nvals)
 
-            width <- box.width # box.ratio/(1 + box.ratio)
+            width <- # box.ratio / (1 + box.ratio) by default
+                rep(box.width, length.out = length(y))
 
             if (reference)
                 panel.abline(h = origin,
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
 
             for (i in unique(x))
             {
@@ -353,11 +316,10 @@ panel.barchart <-
                                border = border[groups[ok][ord][pos]],
                                lty = lty[groups[ok][ord][pos]],
                                lwd = lwd[groups[ok][ord][pos]],
-                               width = rep(width, nok),
+                               width = width[ok][ord][pos],
                                height = y[ok][ord][pos],
                                just = c("centre", "bottom"),
-                               identifier = paste(identifier, "pos", i,
-                                 sep = "."))
+                               identifier = paste(identifier, "pos", i, sep = "."))
                 neg <- y[ok][ord] < 0
                 nok <- sum(neg, na.rm = TRUE)
                 if (nok > 0)
@@ -367,25 +329,19 @@ panel.barchart <-
                                border = border[groups[ok][ord][neg]],
                                lty = lty[groups[ok][ord][neg]],
                                lwd = lwd[groups[ok][ord][neg]],
-                               width = rep(width, nok),
+                               width = width[ok][ord][neg],
                                height = y[ok][ord][neg],
                                just = c("centre", "bottom"),
-                               identifier = paste(identifier, "neg", i,
-                                 sep = "."))
+                               identifier = paste(identifier, "neg", i, sep = "."))
             }
         }
-        else
+        else # grouped, with side by side bars
         {
             if (is.null(origin))
             {
                 origin <- current.panel.limits()$ylim[1]
                 reference = FALSE
             }
-##             vals <- seq_len(nlevels(groups))
-##             groups <- as.numeric(groupSub(groups, ...))
-##             ## vals <- sort(unique(groups))
-##             nvals <- length(vals)
-
             col <- rep(col, length.out = nvals)
             border <- rep(border, length.out = nvals)
             lty <- rep(lty, length.out = nvals)
@@ -397,8 +353,7 @@ panel.barchart <-
                              col = reference.line$col,
                              lty = reference.line$lty,
                              lwd = reference.line$lwd,
-                             identifier = paste(identifier, "abline",
-                               sep = "."))
+                             identifier = paste(identifier, "abline", sep = "."))
             for (i in unique(x))
             {
                 ok <- x == i
@@ -412,8 +367,7 @@ panel.barchart <-
                            width = rep(width, nok),
                            height = y[ok] - origin,
                            just = c("centre", "bottom"),
-                           identifier = paste(identifier, "x", i,
-                               sep = "."))
+                           identifier = paste(identifier, "x", i, sep = "."))
             }
         }
     }
@@ -431,6 +385,7 @@ panel.dotplot <-
              levels.fos = if (horizontal) unique(y) else unique(x),
              groups = NULL,
              ...,
+             grid = lattice.getOption("default.args")$grid,
              identifier = "dotplot")
 {
     x <- as.numeric(x)
@@ -442,6 +397,13 @@ panel.dotplot <-
 
     if (horizontal)
     {
+        if (!isFALSE(grid))
+        {
+            if (!is.list(grid))
+                grid <- if (isTRUE(grid)) list(h = 0, v = -1, x = x)
+                        else list(h = 0, v = 0)
+            do.call(panel.grid, grid)
+        }
         panel.abline(h = levels.fos,
                      col = col.line, lty = lty, lwd = lwd,
                      identifier = paste(identifier, "abline", sep="."))
@@ -450,10 +412,18 @@ panel.dotplot <-
                      ## lty = lty, lwd = lwd,
                      groups = groups,
                      horizontal = horizontal, ...,
+                     grid = FALSE,
                      identifier = identifier)
     }
     else
     {
+        if (!isFALSE(grid))
+        {
+            if (!is.list(grid))
+                grid <- if (isTRUE(grid)) list(h = -1, v = 0, y = y)
+                        else list(h = 0, v = 0)
+            do.call(panel.grid, grid)
+        }
         panel.abline(v = levels.fos,
                      col = col.line, lty = lty, lwd = lwd,
                      identifier = paste(identifier, "abline", sep="."))
@@ -462,6 +432,7 @@ panel.dotplot <-
                      ## lty = lty, lwd = lwd,
                      groups = groups,
                      horizontal = horizontal, ...,
+                     grid = FALSE,
                      identifier = identifier)
     }
 }
@@ -474,8 +445,19 @@ panel.stripplot <-
     function(x, y, jitter.data = FALSE,
              factor = 0.5, amount = NULL,
              horizontal = TRUE, groups = NULL, ...,
+             grid = lattice.getOption("default.args")$grid,
              identifier = "stripplot")
 {
+    if (!isFALSE(grid))
+    {
+        if (!is.list(grid))
+            grid <- switch(as.character(grid),
+                           "TRUE" = list(h = -1, v = -1, x = x, y = y),
+                           "h" = list(h = -1, v = 0, y = y),
+                           "v" = list(h = 0, v = -1, x = x),
+                           list(h = 0, v = 0))
+        do.call(panel.grid, grid)
+    }
     if (!any(is.finite(x) & is.finite(y))) return()
     panel.xyplot(x = x,
                  y = y,
@@ -484,6 +466,7 @@ panel.stripplot <-
                  factor = factor, amount = amount,
                  groups = groups,
                  horizontal = horizontal, ...,
+                 grid = FALSE,
                  identifier = identifier)
 }
 
@@ -608,7 +591,7 @@ panel.bwplot <-
 
         ## dot
 
-        if (all(pch == "|"))
+        if (all(!is.na(pch) & pch == "|"))
         {
             mult <- if (notch) 1 - notch.frac else 1
             panel.segments(blist.stats[, 3],
@@ -727,7 +710,7 @@ panel.bwplot <-
 
         ## dot
 
-        if (all(pch == "|"))
+        if (all(!is.na(pch) & pch == "|"))
         {
             mult <- if (notch) 1 - notch.frac else 1
             panel.segments(levels.fos - mult * blist.height / 2,
@@ -770,16 +753,6 @@ panel.bwplot <-
 
 
 
-
-
-
-
-
-
-
-
-
-
 panel.violin <-
     function(x, y, box.ratio = 1, box.width = box.ratio / (1 + box.ratio),
              horizontal = TRUE,
@@ -829,18 +802,31 @@ panel.violin <-
     darg$to <- to
     darg$cut <- cut
     darg$na.rm <- na.rm
-    my.density <- function(x)
-    {
-        ans <- try(do.call("density", c(list(x = x), darg)), silent = TRUE)
-        ## if (inherits(ans, "try-error")) list(x = numeric(0), y = numeric(0)) else ans
-        if (inherits(ans, "try-error"))
-            list(x = rep(x[1], 3),
-                 y = c(0, 1, 0))
-        else ans
+
+    .density <- function(x, density.args) {
+        if (stats::sd(x, na.rm = TRUE) > 0)
+            do.call(stats::density, c(list(x = x), density.args))
+        else
+            list(x = rep(x[1], 3), y = c(0, 1, 0))
     }
     numeric.list <- if (horizontal) split(x, factor(y)) else split(y, factor(x))
+    # Recycle arguments
+    # Add index to ensure that arguments are multiple of number of groups
+    darg$index <- seq_along(numeric.list)
+    darg <- tryCatch({
+        do.call(data.frame, darg)
+    }, error = function(e) {
+        darg$index <- NULL
+        stop(gettextf("length of '%s' must be  1 or multiple of group length (%d)",
+                      paste0(names(darg), collapse = ', '),
+                      length(numeric.list)))
+    })
+    darg$index <- NULL
+
     levels.fos <- as.numeric(names(numeric.list))
-    d.list <- lapply(numeric.list, my.density)
+    d.list <- lapply(seq_along(numeric.list), function(i) {
+        .density(numeric.list[[i]], darg[i, ])
+    })
     ## n.list <- sapply(numeric.list, length)  UNNECESSARY
     dx.list <- lapply(d.list, "[[", "x")
     dy.list <- lapply(d.list, "[[", "y")
@@ -849,6 +835,27 @@ panel.violin <-
     if (varwidth) max.d[] <- max(max.d)
 
     ##str(max.d)
+    # Plot arguments either 1 or multiple of number of groups
+    plot.arg <- list()
+    plot.arg$alpha <- alpha
+    # Map fill = col, col = border for gpar call
+    plot.arg$fill <- col
+    plot.arg$col <- border
+    plot.arg$lty <- lty
+    plot.arg$lwd <- lwd
+
+    plot.arg$index <- seq_along(numeric.list)
+
+    plot.arg <- tryCatch({
+      do.call(data.frame, plot.arg)
+      }, error = function(e) {
+        plot.arg$index <- NULL
+        stop(sprintf('%s must be length 1 or a vector of 
+             length multiple of group length (%d)',
+                     paste0(names(plot.arg), collapse = ', '),
+                     length(numeric.list)))
+      })
+    plot.arg$index <- NULL
 
     cur.limits <- current.panel.limits()
     xscale <- cur.limits$xlim
@@ -875,7 +882,7 @@ panel.violin <-
                              default.units = "native",
                              name = trellis.grobname(identifier,
                                type = "panel", group = group),
-                             gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+                             gp = do.call(gpar, plot.arg[i, ]))
                 popViewport()
             }
         }
@@ -895,7 +902,7 @@ panel.violin <-
                              default.units = "native",
                              name = trellis.grobname(identifier,
                                type = "panel", group = group),
-                             gp = gpar(fill = col, col = border, lty = lty, lwd = lwd, alpha = alpha))
+                             gp = do.call(gpar, plot.arg[i, ]))
                 popViewport()
             }
         }
@@ -910,6 +917,12 @@ panel.violin <-
 
 dotplot <- function(x, data, ...) UseMethod("dotplot")
 
+dotplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(dotplot(formula, x, ...), list(call = ocall))
+}
 
 ## dotplot.numeric <-
 ##     function(formula, data = NULL, xlab = deparse(substitute(formula)), ...)
@@ -933,7 +946,7 @@ dotplot <- function(x, data, ...) UseMethod("dotplot")
 dotplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(dotplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -952,7 +965,7 @@ dotplot.table <-
     function(x, data = NULL, groups = TRUE,
              ..., horizontal = TRUE)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
     if (!is.null(data)) warning("explicit 'data' specification ignored")
     data <- as.data.frame(x)
     nms <- names(data)
@@ -981,9 +994,23 @@ dotplot.table <-
 }
 
 
-dotplot.default <- function(x, data = NULL, ...) dotplot(table(x), data, ...)
-dotplot.array <- function(x, data = NULL, ...) dotplot(as.table(x), data, ...)
-dotplot.matrix <- function(x, data = NULL, ...) dotplot(as.table(x), data, ...)
+dotplot.default <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
+    modifyList(dotplot(table(x), data, ...), list(call = ocall))
+}
+
+dotplot.array <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
+    modifyList(dotplot(as.table(x), data, ...), list(call = ocall))
+}
+
+dotplot.matrix <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
+    modifyList(dotplot(as.table(x), data, ...), list(call = ocall))
+}
 
 
 dotplot.formula <-
@@ -993,7 +1020,7 @@ dotplot.formula <-
              default.prepanel = lattice.getOption("prepanel.default.dotplot"),
              ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(dotplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(dotplot)
     ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
@@ -1007,11 +1034,17 @@ dotplot.formula <-
 
 barchart <- function(x, data, ...) UseMethod("barchart")
 
+barchart.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(barchart(formula, x, ...), list(call = ocall))
+}
 
 barchart.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -1031,7 +1064,7 @@ barchart.table <-
     function(x, data = NULL, groups = TRUE,
              origin = 0, stack = TRUE, ..., horizontal = TRUE)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
     if (!is.null(data)) warning("explicit 'data' specification ignored")
     data <- as.data.frame(x)
     nms <- names(data)
@@ -1058,13 +1091,26 @@ barchart.table <-
                         ##groups = groups,
                         origin = origin, stack = stack, 
                         ...),
-               list(ocall = ocall))
+               list(call = ocall))
 }
 
-barchart.default <- function(x, data = NULL, ...) barchart(table(x), data, ...)
-barchart.array <- function(x, data = NULL, ...) barchart(as.table(x), data, ...)
-barchart.matrix <- function(x, data = NULL, ...) barchart(as.table(x), data, ...)
+barchart.default <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
+    modifyList(barchart(table(x), data, ...), list(call = ocall))
+}
 
+barchart.array <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
+    modifyList(barchart(as.table(x), data, ...), list(call = ocall))
+}
+
+barchart.matrix <- function(x, data = NULL, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
+    modifyList(barchart(as.table(x), data, ...), list(call = ocall))
+}
 
 barchart.formula <-
     function(x,
@@ -1074,7 +1120,7 @@ barchart.formula <-
              box.ratio = 2, 
              ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(barchart)
+    ocall <- sys.call(); ocall[[1]] <- quote(barchart)
     ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
@@ -1089,11 +1135,17 @@ barchart.formula <-
 
 stripplot <- function(x, data, ...)  UseMethod("stripplot")
 
+stripplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(stripplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(stripplot(formula, x, ...), list(call = ocall))
+}
 
 stripplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(stripplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(stripplot)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -1115,7 +1167,7 @@ stripplot.formula <-
              default.prepanel = lattice.getOption("prepanel.default.stripplot"),
              ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(stripplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(stripplot)
     ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
@@ -1133,12 +1185,19 @@ stripplot.formula <-
 
 bwplot <- function(x, data, ...) UseMethod("bwplot")
 
+bwplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(bwplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(bwplot(formula, x, ...), list(call = ocall))
+}
+
 
 
 bwplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(bwplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(bwplot)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -1157,7 +1216,7 @@ bwplot.formula <-
              data = NULL,
              allow.multiple = is.null(groups) || outer,
              outer = FALSE,
-             auto.key = FALSE,
+             auto.key = lattice.getOption("default.args")$auto.key,
              aspect = "fill",
              panel = lattice.getOption("panel.bwplot"),
              prepanel = NULL,
@@ -1259,7 +1318,7 @@ bwplot.formula <-
     ## less complicated components:
 
     foo <-
-        do.call("trellis.skeleton",
+        do.call(trellis.skeleton,
                 c(list(formula = formula,
                        cond = cond,
                        aspect = aspect,
@@ -1274,13 +1333,13 @@ bwplot.formula <-
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(bwplot)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(bwplot)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
     if (is.character(scales)) scales <- list(relation = scales)
     scales <- updateList(default.scales, scales)
-    foo <- c(foo, do.call("construct.scales", scales))
+    foo <- c(foo, do.call(construct.scales, scales))
 
     ## Step 3: Decide if limits were specified in call:
 
@@ -1453,26 +1512,12 @@ bwplot.formula <-
     if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
-            list(list(fun = "drawSimpleKey",
-                      args =
-                      updateList(list(text = levels(as.factor(groups)),
-                                      points = if (is.standard.barchart) FALSE else TRUE,
-                                      rectangles = if (is.standard.barchart) TRUE else FALSE,
-                                      lines = FALSE), 
-                                 if (is.list(auto.key)) auto.key else list())))
-        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
-        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
-        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
-
-        names(foo$legend) <- 
-            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
-                "inside"
-            else
-                "top"
-        if (!is.null(foo$legend[[1]]$args$space))
-            names(foo$legend) <- foo$legend[[1]]$args$space
+            autoKeyLegend(list(text = levels(as.factor(groups)),
+                               points = if (is.standard.barchart) FALSE else TRUE,
+                               rectangles = if (is.standard.barchart) TRUE else FALSE,
+                               lines = FALSE),
+                          auto.key)
     }
-
     class(foo) <- "trellis"
     foo
 }
