@@ -103,8 +103,19 @@ panel.densityplot <-
              jitter.amount = 0.01 * diff(current.panel.limits()$ylim),
              type = "p",
              ...,
+             grid = lattice.getOption("default.args")$grid,
              identifier = "density")
 {
+    if (!isFALSE(grid))
+    {
+        if (!is.list(grid))
+            grid <- switch(as.character(grid),
+                           "TRUE" = list(h = -1, v = -1, x = x),
+                           "h" = list(h = -1, v = 0),
+                           "v" = list(h = 0, v = -1, x = x),
+                           list(h = 0, v = 0))
+        do.call(panel.grid, grid)
+    }
     if (ref)
     {
         reference.line <- trellis.par.get("reference.line")
@@ -170,11 +181,18 @@ panel.densityplot <-
 
 densityplot <- function(x, data, ...) UseMethod("densityplot")
 
+densityplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(densityplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(densityplot(formula, x, ...), list(call = ocall))
+}
+
 
 densityplot.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(densityplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(densityplot)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -198,7 +216,7 @@ densityplot.formula <-
              data = NULL,
              allow.multiple = is.null(groups) || outer,
              outer = !is.null(groups),
-             auto.key = FALSE,
+             auto.key = lattice.getOption("default.args")$auto.key,
              aspect = "fill",
              panel = lattice.getOption("panel.densityplot"),
              prepanel = NULL,
@@ -301,7 +319,7 @@ densityplot.formula <-
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(densityplot)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(densityplot)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -402,30 +420,15 @@ densityplot.formula <-
           cond.orders(foo))
     foo[names(more.comp)] <- more.comp
 
-
     if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
-            list(list(fun = "drawSimpleKey",
-                      args =
-                      updateList(list(text = levels(as.factor(groups)),
-                                      points = FALSE,
-                                      rectangles = FALSE,
-                                      lines = TRUE),
-                                 if (is.list(auto.key)) auto.key else list())))
-        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
-        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
-        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
-
-        names(foo$legend) <- 
-            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
-                "inside"
-            else
-                "top"
-        if (!is.null(foo$legend[[1]]$args$space))
-            names(foo$legend) <- foo$legend[[1]]$args$space
+            autoKeyLegend(list(text = levels(as.factor(groups)),
+                               points = FALSE,
+                               rectangles = FALSE,
+                               lines = TRUE),
+                          auto.key)
     }
-
     class(foo) <- "trellis"
     foo
 }
