@@ -40,6 +40,10 @@ hist.constructor <-
         hist(as.numeric(x), breaks = breaks, right = right, plot = FALSE)
 }
 
+    ## if (prefer.density && type != "density" && !is.function(breaks))
+    ##     warning(gettextf("type='%s' can be misleading in this context", type))
+
+
 
 prepanel.default.histogram <-
     function(x,
@@ -61,20 +65,20 @@ prepanel.default.histogram <-
         }
         h <-
             hist.constructor(x, breaks = breaks, ...)
+        if (type != "density" && is.function(breaks))
+        {
+            ## warn if not equispaced breaks
+            if (!isTRUE(all.equal(diff(range(diff(h$breaks))), 0)))
+                warning(gettextf("type='%s' can be misleading in this context", type))
+        }
         y <-
             switch(type,
                    count = h$counts,
                    percent = 100 * h$counts/length(x),
                    density = h$density)
-        ## y <-
-        ##     if (type == "count") h$counts
-        ##     else if (type == "percent") 100 * h$counts / length(x)
-        ##     else h$density
         list(xlim =
-             if (is.factor(x)) levels(x)
-             else scale.limits(c(x, h$breaks)),
-             ## if (is.factor(x)) levels(x)
-             ## else range(x, h$breaks, finite = TRUE),
+             if (is.factor(x)) levels(x) # same as scale_limits(x)
+             else scale_limits(c(x, h$breaks)),
              ylim = range(0, y, finite = TRUE),
              dx = 1,
              dy = 1)
@@ -153,11 +157,18 @@ panel.histogram <-
 
 histogram <- function(x, data, ...) UseMethod("histogram")
 
+histogram.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(histogram)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(histogram(formula, x, ...), list(call = ocall))
+}
+
 
 histogram.factor <- histogram.numeric <-
     function(x, data = NULL, xlab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(histogram)
+    ocall <- sys.call(); ocall[[1]] <- quote(histogram)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -179,7 +190,7 @@ histogram.formula <-
              data = NULL,
              allow.multiple = is.null(groups) || outer,
              outer = TRUE,
-             auto.key = FALSE,
+             auto.key = lattice.getOption("default.args")$auto.key,
              aspect = "fill",
              panel = lattice.getOption("panel.histogram"),
              prepanel = NULL,
@@ -257,11 +268,12 @@ histogram.formula <-
                        ylab = ylab,
                        xlab.default = form$right.name,
                        ylab.default = "dummy",
-                       lattice.options = lattice.options), dots))
+                       lattice.options = lattice.options), dots),
+                quote = TRUE)
                           
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(histogram)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(histogram)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -310,19 +322,37 @@ histogram.formula <-
     ## frequency histogram is going to be misleading.
 
     if (missing(breaks)) # explicit NULL, or function, or character is fine
-        breaks <- # use nint and endpoints
-            if (is.factor(x)) seq_len(1 + nlevels(x)) - 0.5
-            else do.breaks(as.numeric(endpoints), nint)
+    {
+        breaks <- lattice.getOption("histogram.breaks")
+        if (is.null(breaks)) # nothing specified
+            breaks <- # use nint and endpoints
+                if (is.factor(x)) seq_len(1 + nlevels(x)) - 0.5
+                else do.breaks(as.numeric(endpoints), nint)
+    }
 
+    ## We would like to issue a warning if type!="density" but breaks
+    ## are not equispaced.  This is done here: BUT there is a problem
+    ## if 'breaks' is a function, because then we cannot know if the
+    ## breaks are equispaced here.  On the other hand, we can issue
+    ## the warning in the prepanel/panel function, but that could
+    ## cause unnecessary repititions.
+
+    ## The compromise is that we only issue a warning when 'breaks' is
+    ## not a function (even though we still prefer type=density in
+    ## that case because we don't want to impose a default of
+    ## equispaced breaks in that case).  Ideally, the prepanel
+    ## function should check if type!=density and warn if necessary
+    ## when breaks is a function.
+    
     prefer.density <- 
         (is.function(breaks) || 
          (is.null(breaks) && !equal.widths) ||
-         (is.numeric(breaks) && !isTRUE(all.equal(diff(range(diff(breaks))), 0)))
+         (is.numeric(breaks) && (length(breaks) > 1) && !isTRUE(all.equal(diff(range(diff(breaks))), 0)))
          )
     if (missing(type) && prefer.density)
         type <- "density"
     type <- match.arg(type)
-    if (prefer.density && type != "density")
+    if (prefer.density && type != "density" && !is.function(breaks))
         warning(gettextf("type='%s' can be misleading in this context", type))
 
     ## this is normally done earlier (in trellis.skeleton), but in
@@ -392,30 +422,15 @@ histogram.formula <-
                    cond.orders(foo))
     foo[names(more.comp)] <- more.comp
 
-
     if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
-            list(list(fun = "drawSimpleKey",
-                      args =
-                      updateList(list(text = levels(as.factor(groups)),
-                                      points = FALSE,
-                                      rectangles = TRUE,
-                                      lines = FALSE),
-                                 if (is.list(auto.key)) auto.key else list())))
-        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
-        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
-        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
-
-        names(foo$legend) <- 
-            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
-                "inside"
-            else
-                "top"
-        if (!is.null(foo$legend[[1]]$args$space))
-            names(foo$legend) <- foo$legend[[1]]$args$space
+            autoKeyLegend(list(text = levels(as.factor(groups)),
+                               points = FALSE,
+                               rectangles = TRUE,
+                               lines = FALSE),
+                          auto.key)
     }
-
     class(foo) <- "trellis"
     foo
 }
