@@ -1,6 +1,6 @@
 
 
-### Copyright (C) 2001-2006  Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
+### Copyright (C) 2001-2023  Deepayan Sarkar <Deepayan.Sarkar@R-project.org>
 ###
 ### This file is part of the lattice package for R.
 ### It is made available under the terms of the GNU General Public
@@ -19,8 +19,6 @@
 ### MA 02110-1301, USA
 
 
-
-
 level.colors <- function(x, at, col.regions, colors = TRUE, ...)
 {
     ind.col <- cut(x, at, include.lowest = TRUE, labels = FALSE)
@@ -122,12 +120,14 @@ panel.levelplot <-
              border.lty = 1,
              border.lwd = 0.1,
              ...,
+             region.type = c("grid", "contour"),
              col.regions = regions$col,
              alpha.regions = regions$alpha,
              identifier = "levelplot")
 {
     if (length(subscripts) == 0) return()
     regions <- trellis.par.get("regions")
+    region.type <- match.arg(region.type)
     label.style <- match.arg(label.style)
     x.is.factor <- is.factor(x)
     y.is.factor <- is.factor(y)
@@ -238,20 +238,47 @@ panel.levelplot <-
     idx <- match(x, ux)
     idy <- match(y, uy)
 
-    if (region) 
-        grid.rect(x = cx[idx],
-                  y = cy[idy],
-                  width = lx[idx] * scaleWidth(z, shrinkx[1], shrinkx[2], fullZrange),
-                  height = ly[idy] * scaleWidth(z, shrinky[1], shrinky[2], fullZrange),
-                  default.units = "native",
-                  name = trellis.grobname(paste(identifier, "rect", sep="."),
-                    type = "panel", group = group),
-                  gp =
-                  gpar(fill = zcol,
-                       col = border,
-                       lwd = border.lwd,
-                       lty = border.lty,
-                       alpha = alpha.regions))
+    if (region)
+    {
+        if (region.type == "grid")
+        {
+            grid.rect(x = cx[idx],
+                      y = cy[idy],
+                      width = lx[idx] * scaleWidth(z, shrinkx[1], shrinkx[2], fullZrange),
+                      height = ly[idy] * scaleWidth(z, shrinky[1], shrinky[2], fullZrange),
+                      default.units = "native",
+                      name = trellis.grobname(paste(identifier, "rect", sep="."),
+                                              type = "panel", group = group),
+                      gp =
+                          gpar(fill = zcol,
+                               col = border,
+                               lwd = border.lwd,
+                               lty = border.lty,
+                               alpha = alpha.regions))
+        }
+        else if (region.type == "contour")
+        {
+            numcol <- length(at) - 1
+            cols <- level.colors(x = seq_len(numcol) - 0.5,
+                                 at = seq_len(numcol + 1) - 1,
+                                 col.regions = col.regions,
+                                 colors = TRUE)
+            filledContour(x = cx,
+                          y = cy,
+                          z = matrix(z, length(cx)), 
+                          s = at, 
+                          cols = cols,
+                          name = trellis.grobname(paste(identifier, 
+                                                        "polygon", 
+                                                        sep = "."),
+                                                  type = "panel", 
+                                                  group = group),
+                          border = border,
+                          lwd = border.lwd,
+                          lty = border.lty,
+                          alpha = alpha.regions)
+        }
+    } 
 
     if (contour)
     {
@@ -390,10 +417,18 @@ panel.levelplot <-
 
 contourplot <- function(x, data, ...) UseMethod("contourplot")
 
+contourplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(contourplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(contourplot(formula, x, ...), list(call = ocall))
+}
+
 
 contourplot.table <-
     function(x, data = NULL, aspect = "iso", ..., xlim, ylim)
 {
+    ocall <- sys.call(); ocall[[1]] <- quote(contourplot)
     ## see comments in levelplot.table below
     if (!missing(data)) warning("explicit 'data' specification ignored")
     dn <- dimnames(x)
@@ -413,8 +448,9 @@ contourplot.table <-
     }
     if (missing(xlim)) xlim <- dn[[1]]
     if (missing(ylim)) ylim <- dn[[2]]
-    contourplot(as.formula(form), data,
-                aspect = aspect, xlim = xlim, ylim = ylim, ...)
+    modifyList(contourplot(as.formula(form), data,
+                           aspect = aspect, xlim = xlim, ylim = ylim, ...),
+               list(call = ocall))
 }
 
 
@@ -426,6 +462,7 @@ contourplot.matrix <-
 {
     stopifnot(length(row.values) == nrow(x),
               length(column.values) == ncol(x))
+    ocall <- sys.call(); ocall[[1]] <- quote(contourplot)
     if (!missing(data)) warning("explicit 'data' specification ignored")
     form <- z ~ row * column
     data <- expand.grid(row = row.values, column = column.values)
@@ -439,7 +476,8 @@ contourplot.matrix <-
         ylim <-
             if (!is.null(colnames(x))) colnames(x)
             else range(column.values, finite = TRUE) + c(-0.5, 0.5)
-    contourplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...)
+    modifyList(contourplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...),
+               list(call = ocall))
 }
 
 
@@ -462,8 +500,9 @@ contourplot.matrix <-
 
 contourplot.array <- function(x, data = NULL, ...)
 {
+    ocall <- sys.call(); ocall[[1]] <- quote(contourplot)
     if (!missing(data)) warning("explicit 'data' specification ignored")
-    contourplot(as.table(x), ...)
+    modifyList(contourplot(as.table(x), ...), list(call = ocall))
 }
 
 
@@ -479,7 +518,7 @@ contourplot.formula <-
              region = FALSE,
              ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(contourplot)
+    ocall <- sys.call(); ocall[[1]] <- quote(contourplot)
     ccall <- match.call()
     ccall$data <- data
     ccall$panel <- panel
@@ -499,6 +538,13 @@ contourplot.formula <-
 
 levelplot <- function(x, data, ...) UseMethod("levelplot")
 
+levelplot.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(levelplot)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(levelplot(formula, x, ...), list(call = ocall))
+}
+
 levelplot.array <- function(x, data = NULL, ...)
 {
     if (!missing(data)) warning("explicit 'data' specification ignored")
@@ -508,6 +554,7 @@ levelplot.array <- function(x, data = NULL, ...)
 levelplot.table <-
     function(x, data = NULL, aspect = "iso", ..., xlim, ylim)
 {
+    ocall <- sys.call(); ocall[[1]] <- quote(levelplot)
     if (!missing(data)) warning("explicit 'data' specification ignored")
     dn <- dimnames(x) ## cannot be NULL for tables
 
@@ -533,8 +580,9 @@ levelplot.table <-
     ## if rownames/colnames are non-null, use them to label (not the unique versions)
     if (missing(xlim)) xlim <- dn[[1]]
     if (missing(ylim)) ylim <- dn[[2]]
-    levelplot(as.formula(form), data,
-              aspect = aspect, xlim = xlim, ylim = ylim, ...)
+    modifyList(levelplot(as.formula(form), data,
+                         aspect = aspect, xlim = xlim, ylim = ylim, ...),
+               list(call = ocall))
 }
 
 
@@ -546,6 +594,7 @@ levelplot.matrix <-
 {
     stopifnot(length(row.values) == nrow(x),
               length(column.values) == ncol(x))
+    ocall <- sys.call(); ocall[[1]] <- quote(levelplot)
     if (!missing(data)) warning("explicit 'data' specification ignored")
     form <- z ~ row * column
     data <- expand.grid(row = row.values, column = column.values)
@@ -559,7 +608,8 @@ levelplot.matrix <-
         ylim <-
             if (!is.null(colnames(x))) colnames(x)
             else range(column.values, finite = TRUE) + c(-0.5, 0.5)
-    levelplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...)
+    modifyList(levelplot(form, data, aspect = aspect, xlim = xlim, ylim = ylim, ...),
+               list(call = ocall))
 }
 
 ### See FIXME for contourplot.matrix above
@@ -702,12 +752,13 @@ levelplot.formula <-
                        ylab = ylab,
                        xlab.default = form$right.x.name,
                        ylab.default = form$right.y.name,
-                       lattice.options = lattice.options), dots))
+                       lattice.options = lattice.options), dots),
+                quote = TRUE)
 
     
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(levelplot)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(levelplot)
 
     ## Step 2: Compute scales.common (excluding limits)
 
