@@ -20,90 +20,6 @@
 
 
 
-## FIXME: should be retired (quantile is good enough now, but check first)
-
-fast.quantile <- 
-    function(x,
-             probs = seq(0, 1, 0.25),
-             na.rm = FALSE,
-             names = FALSE,
-             type = 7, ...) 
-{
-    if (na.rm) 
-        x <- x[!is.na(x)]
-    else if (any(is.na(x))) 
-        stop("missing values and NaN's not allowed if 'na.rm' is FALSE")
-    if (any((p.ok <- !is.na(probs)) & (probs < 0 | probs > 1))) 
-        stop("'probs' outside [0,1]")
-    n <- length(x)
-    if (na.p <- any(!p.ok)) {
-        o.pr <- probs
-        probs <- probs[p.ok]
-    }
-    np <- length(probs)
-    if (n > 0 && np > 0) {
-        if (type == 7) {
-            index <- 1 + (n - 1) * probs
-            lo <- floor(index)
-            hi <- ceiling(index)
-            x <- sort(x)
-            i <- index > lo
-            qs <- x[lo]
-            i <- seq_along(i)[i & !is.na(i)]
-            h <- (index - lo)[i]
-            qs[i] <- ifelse(h == 0, qs[i], (1 - h) * qs[i] + 
-                h * x[hi[i]])
-        }
-        else {
-            if (type <= 3) {
-                nppm <- if (type == 3) 
-                  n * probs - 0.5
-                else n * probs
-                j <- floor(nppm)
-                switch(type, h <- ifelse(nppm > j, 1, 0), h <- ifelse(nppm > 
-                  j, 1, 0.5), h <- ifelse((nppm == j) & ((j%%2) == 
-                  0), 0, 1))
-            }
-            else {
-                switch(type - 3, {
-                  a <- 0
-                  b <- 1
-                }, a <- b <- 0.5, a <- b <- 0, a <- b <- 1, a <- b <- 1/3, 
-                  a <- b <- 3/8)
-                fuzz <- 4 * .Machine$double.eps
-                nppm <- a + probs * (n + 1 - a - b)
-                j <- floor(nppm + fuzz)
-                h <- nppm - j
-                h <- ifelse(abs(h) < fuzz, 0, h)
-            }
-            x <- sort(x)
-            x <- c(x[1], x[1], x, x[n], x[n])
-            qs <- ifelse(h == 0, x[j + 2], ifelse(h == 1, x[j + 
-                3], (1 - h) * x[j + 2] + h * x[j + 3]))
-        }
-    }
-    else {
-        qs <- rep(NA_real_, np)
-    }
-    if (names && np > 0) {
-        dig <- max(2, getOption("digits"))
-        names(qs) <- paste(if (np < 100) 
-            formatC(100 * probs, format = "fg", width = 1, digits = dig)
-        else format(100 * probs, trim = TRUE, digits = dig), 
-            "%", sep = "")
-    }
-    if (na.p) {
-        o.pr[p.ok] <- qs
-        names(o.pr) <- rep("", length(o.pr))
-        names(o.pr)[p.ok] <- names(qs)
-        o.pr
-    }
-    else qs
-}
-
-
-
-
 
 prepanel.default.qqmath <-
     function(x,
@@ -162,8 +78,8 @@ prepanel.default.qqmath <-
     {
         xx <- getxx(x, f.value, nobs)
         yy <- getyy(x, f.value, nobs)
-        list(xlim = scale.limits(xx), # range(xx, finite = TRUE),
-             ylim = scale.limits(yy), # range(yy, finite = TRUE),
+        list(xlim = scale_limits(xx), # range(xx, finite = TRUE),
+             ylim = scale_limits(yy), # range(yy, finite = TRUE),
              dx = diff(xx),
              dy = diff(yy))
     }
@@ -235,12 +151,19 @@ panel.qqmath <-
 
 qqmath <- function(x, data, ...) UseMethod("qqmath")
 
+qqmath.data.frame <- function(x, data = NULL, formula = data, ...)
+{
+    ocall <- sys.call(); ocall[[1]] <- quote(qqmath)
+    if (!inherits(formula, "formula")) stop("'formula' must be a formula object")
+    modifyList(qqmath(formula, x, ...), list(call = ocall))
+}
+
 
 
 qqmath.numeric <-
     function(x, data = NULL, ylab = deparse(substitute(x)), ...)
 {
-    ocall <- sys.call(sys.parent()); ocall[[1]] <- quote(qqmath)
+    ocall <- sys.call(); ocall[[1]] <- quote(qqmath)
     ccall <- match.call()
     if (!is.null(ccall$data)) 
         warning("explicit 'data' specification ignored")
@@ -262,7 +185,7 @@ qqmath.formula <-
              outer = !is.null(groups),
              distribution = qnorm,
              f.value = NULL,
-             auto.key = FALSE,
+             auto.key = lattice.getOption("default.args")$auto.key,
              aspect = "fill",
              panel = lattice.getOption("panel.qqmath"),
              prepanel = NULL,
@@ -332,12 +255,12 @@ qqmath.formula <-
                        ylab = ylab,
                        xlab.default = dist.name,
                        ylab.default = form$right.name,
-                       lattice.options = lattice.options),
-                  dots))
+                       lattice.options = lattice.options), dots),
+                quote = TRUE)
 
     dots <- foo$dots # arguments not processed by trellis.skeleton
     foo <- foo$foo
-    foo$call <- sys.call(sys.parent()); foo$call[[1]] <- quote(qqmath)
+    foo$call <- sys.call(); foo$call[[1]] <- quote(qqmath)
 
     ## Step 2: Compute scales.common (leaving out limits for now)
 
@@ -450,24 +373,11 @@ qqmath.formula <-
     if (is.null(foo$legend) && needAutoKey(auto.key, groups))
     {
         foo$legend <-
-            list(list(fun = "drawSimpleKey",
-                      args =
-                      updateList(list(text = levels(as.factor(groups)),
-                                      points = TRUE,
-                                      rectangles = FALSE,
-                                      lines = FALSE),
-                                 if (is.list(auto.key)) auto.key else list())))
-        foo$legend[[1]]$x <- foo$legend[[1]]$args$x
-        foo$legend[[1]]$y <- foo$legend[[1]]$args$y
-        foo$legend[[1]]$corner <- foo$legend[[1]]$args$corner
-
-        names(foo$legend) <- 
-            if (any(c("x", "y", "corner") %in% names(foo$legend[[1]]$args)))
-                "inside"
-            else
-                "top"
-        if (!is.null(foo$legend[[1]]$args$space))
-            names(foo$legend) <- foo$legend[[1]]$args$space
+            autoKeyLegend(list(text = levels(as.factor(groups)),
+                               points = TRUE,
+                               rectangles = FALSE,
+                               lines = FALSE),
+                          auto.key)
     }
     class(foo) <- "trellis"
     foo
